
/* 
 * Codifica di Huffman (6/06/2021)
 *
 * Classe Node
 * nodi dell'albero di Huffman
 *
 * implements Comparable<Node>:
 * le istanze di Node possono essere ordinate in una coda con priorità
 */

public class Node  implements Comparable<Node> {


  private static final int NO_BYTE = -1;
  
  
  private final int itm;                   // byte
  
  private final int wgt;                   // peso
  private final int cst;                   // costo: solo ai fini dell'analisi (ottimalità)
  
  private final Node lft, rgt;             // figli sinistro e destro
  
  
  // Costruttori: differenziati per il tipo degli argomenti
  
  public Node( int item, int weight ) {    // nodi foglia
    
    itm = item;
    wgt = weight;
    cst = 0;
    lft = null;
    rgt = null;
  }
  
  public Node( Node left, Node right ) {   // nodi interni
    
    itm = NO_BYTE;
    wgt = left.weight() + right.weight();
    cst = left.cost() + right.cost() + wgt;
    lft = left;
    rgt = right;
  }
  
  
  // Il nodo e' una foglia dell'albero di Huffman?
  
  public boolean isLeaf() {
    
    return ( lft == null );
  }
  
  
  // Metodi per acquisire le informazioni relative ai nodi
  
  public int getByte() {
    
    return itm;                           // byte (nodi foglia)
  }
  
  public int weight() {
    
    return wgt;                           // peso
  }
  
  public int cost() {
    
    return cst;                           // impatto
  }
  
  public Node left() {
    
    return lft;                           // figlio sinistro
  }
  
  public Node right() {
    
    return rgt;                           // figlio destro
  }
  
  
  // Confronto dei pesi dei nodi
  // in accordo alle specifiche per la gestione in una coda con priorita'
  // (protocollo di Comparable<Node>)
  
  public int compareTo( Node n ) {
  
    if ( wgt == n.weight() ) {            // =
      return  0;
    } else if ( wgt < n.weight() ) {      // <
      return -1;
    } else {                              // >
      return  1;
    }
  }
  
  
  // Rappresentazione testuale di un nodo
  // (non rilevante per il progetto)
  
  public String toString() {
    
    if ( isLeaf() ) {
      if ( (getByte() == '@') || (getByte() == '\\') ) {
        return "\\" + getByte() + ":" + weight();
      } else {
        return getByte() + ":" + weight();
      }
    } else {
      return "@:" + weight();
    }
  }

}  // class Node

